/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controllers.rel

import com.persagy.base.extensions.toJson
import com.persagy.server.datacenter.models.entities.rel.RBdInSyList
import com.persagy.server.datacenter.models.entities.rel.RSyInBd
import com.persagy.server.datacenter.models.entities.rel.RSyInBdList
import com.persagy.server.services.rel.RSyInBdService
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.slf4j.LoggerFactory
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 系统所在建筑
 *
 * @author 付国宝
 */
@Tag(name = "030、关系-系统所在建筑")
@RestController
@RequestMapping("/rel/sy-in-bd")
class RSyInBdController {

    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(RSyInBdController::class.java)
    } // Companion object

    /**
     * 创建系统所在建筑关系
     *
     * @param    rSyInBd     系统和建筑的关系对象
     * @return               创建的结果信息
     */
    @Operation(summary = "创建系统所在建筑关系", description = "必填项：SysId（系统id）、BuildingId（建筑id）")
    @PostMapping(value = ["/link"])
    fun create(@RequestBody rSyInBd: RSyInBd): SCreateResponse<RSyInBd> {
        logger.debug("build = ${rSyInBd.toJson()}")
        return RSyInBdService.create(rSyInBd)
    } //Function create()

    /**
     * 根据对象删除
     *
     * @param    rSyInBd      对象
     * @return                删除的结果信息
     */
    @Operation(summary = "根据对象删除系统所在建筑关系，只针对一个对象", description = "必填项：SysId（系统id）、BuildingId（建筑id）")
    @PostMapping(value = ["/unlink"])
    fun delete(@RequestBody rSyInBd: RSyInBd): SBaseResponse {
        return RSyInBdService.deletes(rSyInBd)
    } // Function delete()

    /**
     * 创建系统所在建筑关系
     *
     * @param    rSyInBd     系统和建筑的关系对象
     * @return               创建的结果信息
     */
    @Operation(summary = "系统所在建筑，系统一对多建筑", description = "必填项：SysId（系统id）、BuildingIdList（建筑id列表）,测方法会覆盖以前的记录")
    @PostMapping(value = ["/link-bd"])
    fun createBd(@RequestBody rSyInBd: RSyInBdList): SCreateResponse<RSyInBd>{
        logger.debug("build = ${rSyInBd.toJson()}")
        return RSyInBdService.createSy(rSyInBd)
    } // Function createBd()

    /**
     * 创建所在建筑的系统关系
     *
     * @param    rSyInBd      系统和建筑的关系对象
     * @return                创建的结果信息
     */
    @Operation(summary = "所在建筑的系统，建筑一对多系统", description = "必填项：sysIdList（系统id列表）、BuildingId（建筑id）,测方法会覆盖以前的记录")
    @PostMapping(value = ["/link-sy"])
    fun createSy(@RequestBody rSyInBd: RBdInSyList): SCreateResponse<RSyInBd>{
        logger.debug("build = ${rSyInBd.toJson()}")
        return  RSyInBdService.createsBd(rSyInBd)
    } // Function createSy()

    /**
     * 创建系统所在建筑关系
     *
     * @param    rSyInBd     系统和建筑的关系对象
     * @return               创建的结果信息
     */
    @Operation(summary = "系统所在建筑，系统一对多建筑,此方法不会覆盖", description = "必填项：SysId（系统id）、BuildingIdList（建筑id列表）")
    @PostMapping(value = ["/link-bd-sy"])
    fun createSyBd(@RequestBody rSyInBd: RSyInBdList): SCreateResponse<RSyInBd>{
        logger.debug("build = ${rSyInBd.toJson()}")
        return RSyInBdService.createSy(rSyInBd)
    } // Function createBd()

}