/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.objects

import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.database.SQueryBuilder
import com.persagy.database.extensions.keyValue
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.assistant.ObjInfos
import com.persagy.server.datacenter.models.entities.graphtype.RelationTypeProject
import com.persagy.server.datacenter.models.entities.objects.Equipment
import com.persagy.server.datacenter.models.entities.objects.Property
import com.persagy.server.scanbuilding.UserLog
import com.persagy.server.services.assistant.ObjInfoService
import com.persagy.server.services.base.Service
import com.persagy.server.services.log.DataCenterLogsService
import com.persagy.server.utils.IdUtils
import com.persagy.server.utils.ShaftNameUtil
import com.persagy.service.SBaseService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory

/**
 * 设备信息服务
 *
 * @author 张维新
 */
object EquipmentService : Service<Equipment>(SMybatisDao(Equipment::class.java)) {

    private val logger = LoggerFactory.getLogger(EquipmentService::class.java)

    /** 关系总览服务 */
    val relationTypeProjectservice = SBaseService(SMybatisDao(RelationTypeProject::class.java))

    /**
     * 创建前赋值操作
     *
     * @param   equip   设备对象
     */
    override fun onCreateBefore(equip: Equipment): Boolean {
        /** 查询准备类型 */
        if (!equip.id.isNullOrEmpty()) {
            val isStartsWith = equip.id.toString().startsWith("Eq")
            if (!isStartsWith) {
                /** 资产id */
                equip.id = IdUtils.equipmentIdCreate(Opt.projectId!!)
            }
        } else {
            /** 资产id */
            equip.id = IdUtils.equipmentIdCreate(Opt.projectId!!)
        }
        /** 项目id */
        equip.projectId = Opt.projectId
        if (equip.classCode.isNullOrEmpty()) {
            return false
        }
        if(equip.classCode!!.length>6){
            val entity = defClassService.select(
                    SFilter.eq("projectId", equip.projectId!!),
                    SFilter.endWith("code", equip.classCode!!)
            ).entity()
            if (entity!=null){
                equip.classCode = entity.code
            } else {
                return false
            }
        }

        /** 设备名字 */
        if (equip.name.isNullOrEmpty()) {
            val entity = defClassService.select(
                SFilter.eq("projectId", equip.projectId!!),
                SFilter.eq("code", equip.classCode!!)
            ).entity()
            if (entity!=null) {
                equip.name = ShaftNameUtil.shaftName(entity.name+"-")
            }else {
                equip.name = ShaftNameUtil.shaftName("设备-")
            }
        }
        equip.codeType = 1
        return true
    } // Function onCreateBefore()

    override fun onCountSuccess(count: Long) {
        super.onCountSuccess(count)

    }


    /**
     * 查询设备数量之前设置条件
     *
     * @param   queryBuilder    条件信息
     */
    override fun onCountBefore(queryBuilder: SQueryBuilder<Equipment>) {
        queryBuilder.tableName = "v_equipment"
        return
    }  // Function onCreateBefore()

    /**
     * 查询设备之前添加条件方法
     *
     * @param   queryBuilder    条件信息
     */
    override fun onQueryBefore(queryBuilder: SQueryBuilder<Equipment>) {
        queryBuilder.tableName = "v_equipment"
        return
    } // Function onCreateBefore()

    /**
     * 修改设备信息之前的操作
     */
    override fun onUpdateBefore(entity: Equipment): Boolean {
        try {

            if(entity.classCode!!.length>6){
                val defClass = defClassService.select(
                        SFilter.eq("projectId", entity.projectId!!),
                        SFilter.endWith("code", entity.classCode!!)
                ).entity()
                if (defClass!=null){
                    entity.classCode = defClass.code
                } else {
                    return false
                }
            }

            if (entity.propertyId.isNullOrEmpty()) {

//                val property = PropertyService.select(SFilter.eq("equipmentId", entity.id!!)).entity()
//                logger.debug("设备初次关联*********************************")
//                if (property!=null){
//                    property.equipId = null
//                    val list = ArrayList<Property>()
//                    list.add(property)
//                    val sUpdateRequest = SUpdateRequest<Property>()
//                    sUpdateRequest.content = list
//                    sUpdateRequest.projection = arrayListOf("EquipmentId")
//                    PropertyService.updateList(sUpdateRequest)
//                }
            } else {
                val property = PropertyService.select(SFilter.eq("equipId", entity.id!!)).entity()
                if (property != null) {

                    logger.debug("已经有关联的信息*********************************")
                    /** 设备关联的资产如果更改了先取消以前的，在关联新的 */
                    if (entity.propertyId != property.id) {
                        property.equipId = null
                        property.bimId = null
                        val list = ArrayList<Property>()
                        list.add(property)
                        val sUpdateRequest = SUpdateRequest<Property>()
                        sUpdateRequest.content = list
                        sUpdateRequest.projection = arrayListOf("equipId", "bimId")

                        val updateList = PropertyService.updateList(sUpdateRequest)
                        val pr = PropertyService.select(SFilter.eq("id", entity.propertyId!!)).entity()
                        val listp = ArrayList<Property>()
                        pr!!.equipId = entity.id
                        pr.bimId = entity.bimId
                        listp.add(pr)
                        val sUpdate = SUpdateRequest<Property>()
                        sUpdate.content = listp
                        sUpdate.projection = arrayListOf("equipId", "bimId")
                        PropertyService.updateList(sUpdate)
                        logger.debug("现在关联**${sUpdate.toJson()}*******************************")

                        try {
                            /** 处理关系标记 */
                            val listtype = ArrayList<String>()
                            listtype.add("pe2bd")
                            listtype.add("pe2fl")
                            listtype.add("pe2sh")
                            listtype.add("pe2sp")
                            for (type in listtype){
                                val entity = relationTypeProjectservice.select(SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("relationType", type)).entity()
                                if (entity!=null){
                                    entity.computationalState = 2
                                    relationTypeProjectservice.update(entity)
                                }
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                } else {
                    val prop = PropertyService.select(SFilter.eq("id", entity.propertyId!!)).entity()
                    logger.debug("第一次关联***${prop!!.toJson()}******************************")
                    if (prop != null) {
                        prop.equipId = entity.id
                        prop.lastUpdate = null
                        prop.createTime = null
                        prop.name = "1236"
                        val listp = ArrayList<Property>()
                        listp.add(prop)
                        val sUpdate = SUpdateRequest<Property>()
                        sUpdate.content = listp
                        sUpdate.projection = arrayListOf("equipId")
                        val updateList = PropertyService.updateList(sUpdate)
                        logger.debug("第一次更新信息***${sUpdate.toJson()}******************************")
                        logger.debug("第一次更新***${updateList.toJson()}******************************")
                        try {
                            /** 处理关系标记 */
                            val listtype = ArrayList<String>()
                            listtype.add("pe2bd")
                            listtype.add("pe2fl")
                            listtype.add("pe2sh")
                            listtype.add("pe2sp")
                            for (type in listtype){
                                val entity = relationTypeProjectservice.select(SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("relationType", type)).entity()
                                if (entity!=null){
                                    entity.computationalState = 2
                                    relationTypeProjectservice.update(entity)
                                }
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                    }
                }
            }

            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"update-before",entity.objectType!!,
                    entity.id!!,entity,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return true
    } // Function onUpdateBefore()




    /**
     * 删除之前操作
     */
    override fun onDeleteBefore(entity: Equipment): Boolean {
        try {
            val equipment = this.select(SFilter.eq("id", entity.id!!)).entity()
            try {
                /** 日志 */
                DataCenterLogsService.addLogs(Opt.projectId!!,"delete",entity.objectType!!,
                    entity.id!!,entity,Opt.account,Opt.coming)
            } catch (e: Exception) {
                e.printStackTrace()
            }
            /** 清除对应关系 */
            if (equipment != null) {
                val property = PropertyService.select(SFilter.eq("EquipmentId", equipment.id!!)).entity()
                if (property != null && equipment.taskState != null && equipment.taskState == -1 && property.taskState != null && property.taskState == -1) {
                    val list = ArrayList<Property>()
                    list.add(property)
                    PropertyService.deleteByKeysList(list)
                } else {
                    if (property != null) {
                        property.equipId = null
                        property.bimId = null
                        val list = ArrayList<Property>()
                        list.add(property)
                        val sUpdateRequest = SUpdateRequest<Property>()
                        sUpdateRequest.content = list
                        val listStr = ArrayList<String>()
                        listStr.add("EquipmentId")
                        listStr.add("BIMID")
                        sUpdateRequest.projection = listStr
                        PropertyService.updateList(sUpdateRequest)
                    }
                }
            }

        } catch (e: Exception) {
            e.printStackTrace()
        }
        return super.onDeleteBefore(entity)
    } // Function onDeleteBefore()

    /**
     * 查询项目下的设备和部件
     */
    fun queryEquipCom(): SQueryResponse<Equipment> {
        return try {
            val equipList = select(SFilter.eq("projectId", Opt.projectId!!)).exec()
            val sQueryResponse = SQueryResponse<Equipment>(SResponseType.success)
            sQueryResponse.content = equipList
            sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure)
        }
    } // Function queryEquipCom()

    override fun onCreateSuccess(entityList: ArrayList<Equipment>) {
        entityList.forEach {
            try {
                /** 发送消息 */
                rabbitMqService.sendObjects(Opt.projectId!!, Opt.CREATE,it.keyValue("id").toString())
            } catch (e: Exception) {
                e.printStackTrace()
            }
            try {
                val entity = EquipmentService.select(SFilter.eq("id", it.keyValue("id")!!)).entity()
                if (entity != null) {
                    try {
                        /** 日志 */
                        DataCenterLogsService.addLogs(Opt.projectId!!,"create",entity.objectType!!,
                            entity.id!!,entity,Opt.account,Opt.coming)
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                    //添加信息点创建 表号功能号
                    ObjInfoService.createInfo(entity,true,"Eq",entity.classCode!!)
                }

            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
    } //

    /**
     * 更新成功能回调函数
     *
     * @param   entityList      更新成功对象列表
     */
    override fun onUpdateSuccess(entityList: ArrayList<Equipment>) {
        try {
            for (entity in entityList) {
                try {
                    val equip = EquipmentService.select(SFilter.eq("id", entity.keyValue("id")!!)).entity()
                    if (equip != null) {
                        /** 日志 */
                        DataCenterLogsService.addLogs(Opt.projectId!!,"update-after",entity.objectType!!,
                            entity.id!!,entity,Opt.account,Opt.coming)
                        //更新信息点 表号功能号
                        ObjInfoService.createInfo(entity,false,"Eq",entity.classCode!!)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                try {
                    /** 发送消息 */
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.UPDATE,entity.id!!)
                } catch (e: Exception) {
                    e.toString()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

        try {
            for (entity in entityList){
                try {
                    if (!Opt.coming.isNullOrEmpty()){
                        if (Opt.coming == "app"){
                            val user = LocationPointServices.userService.select(SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("phone", Opt.account!!)).entity()
                            val userLog = UserLog()
                            userLog.comming = "app"
                            userLog.action = "更新设备"
                            userLog.id = IdUtils.uuidCreate()
                            userLog.note = "项目ID: "+Opt.projectId+" 用户ID："+user!!.id+" 手机号："+ Opt.account+" 请求来源："+Opt.coming+ " "+entity.toJson()
                            userLog.phone = Opt.account
                            userLog.projectId = Opt.projectId
                            userLog.userId = user.id
                            userLog.name = user.name
                            userLog.objectId = entity.id
                            LocationPointServices.userLogService.insert(userLog)
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }

    } // Function onUpdateSuccess()

    /**
     * @Description:删除成功之后
     */
    override fun onDeleteSuccess(entityList: ArrayList<Equipment>) {
        super.onDeleteSuccess(entityList)
        try {

            val oService = SBaseService(SMybatisDao(ObjInfos::class.java))
            for (entity in entityList) {

                try {
                    rabbitMqService.sendObjects(Opt.projectId!!, Opt.DELETE,entity.id!!)
                } catch (e: Exception) {
                    e.printStackTrace()
                }

                try {//更新信息点 表号功能号
                    oService.delete(SFilter.eq("projectId",Opt.projectId!!), SFilter.eq("objId",entity.id!!), SFilter.eq("objType","Eq"))
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }




} // Object EquipmentService